@@ -33,6 +33,7 @@
 #include "mp_core.h"
 #include "mp_video.h"
 #include "mp_synhi.h"
+#include "mp_lang.h"
 #include "mp_iface.h"
 
 
@@ -124,8 +125,11 @@ static unsigned char * _mpi_ftt_data=
 /* number of lines to move up over current visible line,
    to take into account possible start of comments
    many lines above */
-int _mpi_preread_lines=40;
-
+#if defined(ABSOLUTELY_NO_COLORS) || defined(ANCIENT_HARDWARE)
+int _mpi_preread_lines=0;
+#else
+int _mpi_preread_lines=60;
+#endif
 
 #ifndef ANCIENT_HARDWARE
 #define ANCIENT_HARDWARE 0
@@ -133,6 +137,24 @@ int _mpi_preread_lines=40;
 
 int _mpi_ancient_hardware=ANCIENT_HARDWARE;
 
+/* alternate color set */
+#ifndef ALT_COLOR
+#define ALT_COLOR 0
+#endif
+int mpi_alt_color=ALT_COLOR;
+
+/* use mouse to position cursor or not (if applicable) */
+int mpi_mouse=0;
+
+/* monochrome mode */
+int mpi_monochrome=0;
+
+/* use transparent mode (if applicable) */
+int mpi_transp_mode=1;
+
+/* move selecting flag */
+int mpi_move_selecting=0;
+
 
 /*******************
 	Code
@@ -517,7 +539,7 @@ void mpi_save_as(mp_txt * txt)
 {
 	char * name;
 
-	if((name=mpv_readline(MPR_SAVE,MSG_FILENAME,
+	if((name=mpv_readline(MPR_SAVE,L(MSG_FILENAME),
 		*txt->name=='<' ? NULL : txt->name))==NULL)
 		return;
 
@@ -528,8 +550,8 @@ void mpi_save_as(mp_txt * txt)
 		txt->mod=0;
 	else
 	{
-		mpv_alert(MSG_CANTWRITE, name);
-		mp_name_txt(txt,MSG_UNNAMED);
+		mpv_alert(L(MSG_CANTWRITE), name);
+		mp_name_txt(txt,L(MSG_UNNAMED));
 	}
 }
 
@@ -553,8 +575,8 @@ void mpi_save(mp_txt * txt)
 			txt->mod=0;
 		else
 		{
-			mpv_alert(MSG_CANTWRITE, txt->name);
-			mp_name_txt(txt,MSG_UNNAMED);
+			mpv_alert(L(MSG_CANTWRITE), txt->name);
+			mp_name_txt(txt,L(MSG_UNNAMED));
 		}
 	}
 }
@@ -573,7 +595,7 @@ void mpi_open(char * name, int reopen)
 	mp_txt * txt;
 
 	if(name==NULL &&
-		(name=mpv_readline(MPR_OPEN,MSG_FILENAME,NULL))==NULL)
+		(name=mpv_readline(MPR_OPEN,L(MSG_FILENAME),NULL))==NULL)
 		return;
 
 	if(!reopen && (txt=mp_find_txt(name))!=NULL)
@@ -590,7 +612,7 @@ void mpi_open(char * name, int reopen)
 	}
 	else
 	{
-		mpv_alert(MSG_FILENOTFOUND, name);
+		mpv_alert(L(MSG_FILENOTFOUND), name);
 		mp_delete_txt(_mp_active);
 	}
 
@@ -610,7 +632,7 @@ void mpi_close(mp_txt * txt)
 {
 	if(txt->mod)
 	{
-		if(mpv_confirm(MSG_SAVECHANGES))
+		if(mpv_confirm(L(MSG_SAVECHANGES)))
 			mpi_save(txt);
 	}
 
@@ -634,7 +656,7 @@ void mpi_exec(mp_txt * txt)
 	char * ptr;
 	mp_txt * ctxt;
 
-	if((ptr=mpv_readline(MPR_EXEC,MSG_EXEC,NULL))!=NULL)
+	if((ptr=mpv_readline(MPR_EXEC,L(MSG_EXEC),NULL))!=NULL)
 	{
 		if(*ptr=='|')
 		{
@@ -650,7 +672,7 @@ void mpi_exec(mp_txt * txt)
 					fputc(c,f);
 
 				if(pclose(f)==-1)
-					mpv_alert(MSG_CANTEXEC,ptr);
+					mpv_alert(L(MSG_CANTEXEC),ptr);
 			}
 
 			mp_end_tmp_txt();
@@ -664,12 +686,12 @@ void mpi_exec(mp_txt * txt)
 					mp_put_char(txt,c,1);
 
 				if(pclose(f)==-1)
-					mpv_alert(MSG_CANTEXEC,ptr);
+					mpv_alert(L(MSG_CANTEXEC),ptr);
 			}
 		}
 	}
 	else
-		mpv_alert(MSG_CANTEXEC,ptr);
+		mpv_alert(L(MSG_CANTEXEC),ptr);
 }
 
 
@@ -683,8 +705,9 @@ void mpi_exec(mp_txt * txt)
  * only the local token database is filled and no searching is done.
  * Otherwise (assuming the tag *needs* to be found), the creation of
  * the 'tags' file is forced by executing the 'ctags' program.
+ * Returns -1 on cancellation.
  */
-int mpi_find_tag(char * tag)
+int mpi_find_tag(char * tag, int select_only)
 {
 	FILE * f;
 	char line[4096];
@@ -694,6 +717,7 @@ int mpi_find_tag(char * tag)
 	mp_txt * txt;
 	mp_txt * stxt;
 	int l;
+	int ret;
 
 	if((f=fopen("tags","r"))==NULL)
 	{
@@ -788,49 +812,58 @@ int mpi_find_tag(char * tag)
 	}
 
 	/* show the list */
-	if((l=mpv_list(MSG_TAGLIST,txt))!=-1)
+	ret=0;
+
+	if((l=mpv_list(L(MSG_TAGLIST),txt))!=-1)
 	{
 		/* move to line */
 		mp_move_xy(txt,0,l);
 		mp_move_xy(stxt,0,l);
 
-		/* move to file in the line */
-		mp_move_word_right(txt);
-		mp_get_word(txt,line,sizeof(line));
+		/* save the selected tag into the _draw_word buffer */
+		mp_get_word(txt,_draw_word,sizeof(_draw_word));
 
-		mp_delete_sys_txt(txt);
-
-		/* locate the text and activate, or open */
-		/* open without reopening */
-		mpi_open(line,0);
-
-		/* find now the text to seek */
-		mp_get_str(stxt,line,sizeof(line),'\n');
-
-		/* is it a line number or a string to search? */
-		if(line[0]>='0' && line[0]<='9')
+		if(! select_only)
 		{
-			l=atoi(line)-1;
+			/* move to file in the line */
+			mp_move_word_right(txt);
+			mp_get_word(txt,line,sizeof(line));
 
-			mp_move_xy(_mp_active,0,l);
-		}
-		else
-		{
-			/* copies it into search buffer
-			   to return to tag by pressing ^L */
-			strncpy(_mpi_search_text,line,
-				sizeof(_mpi_search_text));
+			/* locate the text and activate, or open */
+			/* open without reopening */
+			mpi_open(line,0);
 
-			mp_move_bof(_mp_active);
-			mp_seek(_mp_active,line);
+			/* find now the text to seek */
+			mp_get_str(stxt,line,sizeof(line),'\n');
+
+			/* is it a line number or a string to search? */
+			if(line[0]>='0' && line[0]<='9')
+			{
+				l=atoi(line)-1;
+
+				mp_move_xy(_mp_active,0,l);
+			}
+			else
+			{
+				/* copies it into search buffer
+				   to return to tag by pressing ^L */
+				strncpy(_mpi_search_text,line,
+					sizeof(_mpi_search_text));
+
+				mp_move_bof(_mp_active);
+				mp_seek(_mp_active,line);
+			}
 		}
+
+		ret=1;
 	}
 	else
-		mp_delete_sys_txt(txt);
+		ret=-1;
 
+	mp_delete_sys_txt(txt);
 	mp_delete_sys_txt(stxt);
 
-	return(1);
+	return(ret);
 }
 
 
@@ -862,7 +895,7 @@ void mpi_current_list(void)
 	mp_move_left(list);
 	mp_delete_char(list);
 
-	if((l=mpv_list(MSG_OPENLIST,list))!=-1)
+	if((l=mpv_list(L(MSG_OPENLIST),list))!=-1)
 	{
 		mp_move_xy(list, 0, l);
 		mp_get_str(list, tmp, sizeof(tmp), '\n');
@@ -910,7 +943,7 @@ int mpi_insert_template(void)
 	mp_move_left(txt);
 	mp_delete_char(txt);
 
-	if((l=mpv_list(MSG_SELECTTMPL,txt))!=-1)
+	if((l=mpv_list(L(MSG_SELECTTMPL),txt))!=-1)
 	{
 		/* template has been selected: find and insert */
 		_mp_active=t;
@@ -1016,6 +1049,20 @@ int mpi_action(mp_txt * txt, int key)
 			key=MPK_NONE;
 	}
 
+	if(mpi_move_selecting)
+	{
+		if(key==MPK_UP	 || key==MPK_DOWN  ||
+		   key==MPK_LEFT || key==MPK_RIGHT ||
+		   key==MPK_PGUP || key==MPK_PGDN  ||
+		   key==MPK_BOL  || key==MPK_EOL   ||
+		   key==MPK_BOF  || key==MPK_EOF)
+		{
+			if(! mp_marked(txt)) mp_mark(txt);
+		}
+		else
+			mpi_move_selecting=0;
+	}
+
 	switch(key)
 	{
 	case MPK_NONE:
@@ -1097,7 +1144,7 @@ int mpi_action(mp_txt * txt, int key)
 		break;
 
 	case MPK_INSMODE:
-		_mpi_insert=!_mpi_insert;
+		_mpi_insert=! _mpi_insert;
 		break;
 
 	case MPK_MARK:
@@ -1132,21 +1179,21 @@ int mpi_action(mp_txt * txt, int key)
 
 	case MPK_SEEKNEXT:
 		if(!mp_seek(txt,_mpi_search_text))
-			mpv_alert(MSG_NOTFOUND,_mpi_search_text);
+			mpv_alert(L(MSG_NOTFOUND),_mpi_search_text);
 		ret=1;
 
 		break;
 
 	case MPK_REPLACE:
 		if((ptr=mpv_readline(MPR_REPLACETHIS,
-			MSG_REPLACETHIS,NULL))!=NULL)
+			L(MSG_REPLACETHIS),NULL))!=NULL)
 		{
 			strncpy(_mpi_search_text,ptr,sizeof(_mpi_search_text));
 
 			if((ptr=mpv_readline(MPR_REPLACEWITH,
-				MSG_REPLACEWITH,NULL))!=NULL)
+				L(MSG_REPLACEWITH),NULL))!=NULL)
 			{
-				if(mpv_confirm(MSG_TOENDOFFILE))
+				if(mpv_confirm(L(MSG_TOENDOFFILE)))
 				{
 					while(mp_replace(_mp_active,
 						_mpi_search_text,ptr));
@@ -1155,7 +1202,7 @@ int mpi_action(mp_txt * txt, int key)
 				{
 					if(!mp_replace(_mp_active,
 						_mpi_search_text,ptr))
-						mpv_alert(MSG_NOTFOUND,
+						mpv_alert(L(MSG_NOTFOUND),
 						_mpi_search_text);
 				}
 			}
@@ -1167,12 +1214,12 @@ int mpi_action(mp_txt * txt, int key)
 	case MPK_REPLACE_ALL:
 
 		if((ptr=mpv_readline(MPR_REPLACETHIS,
-			MSG_REPLACETHIS,NULL))!=NULL)
+			L(MSG_REPLACETHIS),NULL))!=NULL)
 		{
 			strncpy(_mpi_search_text,ptr,sizeof(_mpi_search_text));
 
 			if((ptr=mpv_readline(MPR_REPLACEWITH,
-				MSG_REPLACEWITH,NULL))!=NULL)
+				L(MSG_REPLACEWITH),NULL))!=NULL)
 			{
 				mp_txt * t;
 
@@ -1218,11 +1265,11 @@ int mpi_action(mp_txt * txt, int key)
 		{
 			mp_txt * txt;
 
-			if((txt=mp_find_txt(MSG_UNNAMED))!=NULL)
+			if((txt=mp_find_txt(L(MSG_UNNAMED)))!=NULL)
 				_mp_active=txt;
 			else
 			{
-				mp_create_txt(MSG_UNNAMED);
+				mp_create_txt(L(MSG_UNNAMED));
 				mps_auto_synhi(_mp_active);
 			}
 		}
@@ -1274,18 +1321,18 @@ int mpi_action(mp_txt * txt, int key)
 
 	case MPK_SEEK:
 
-		if((ptr=mpv_readline(MPR_SEEK,MSG_TEXTTOSEEK,NULL))!=NULL)
+		if((ptr=mpv_readline(MPR_SEEK,L(MSG_TEXTTOSEEK),NULL))!=NULL)
 		{
 			strncpy(_mpi_search_text,ptr,sizeof(_mpi_search_text));
 			if(!mp_seek(_mp_active,_mpi_search_text))
-				mpv_alert(MSG_NOTFOUND,_mpi_search_text);
+				mpv_alert(L(MSG_NOTFOUND),_mpi_search_text);
 		}
 
 		ret=1;
 		break;
 
 	case MPK_GOTO:
-		if((ptr=mpv_readline(MPR_GOTO,MSG_LINETOGO,NULL))!=NULL)
+		if((ptr=mpv_readline(MPR_GOTO,L(MSG_LINETOGO),NULL))!=NULL)
 		{
 			n=atoi(ptr);
 			mp_move_xy(_mp_active,0,n-1);
@@ -1342,9 +1389,9 @@ int mpi_action(mp_txt * txt, int key)
 		{
 			mp_txt * txt;
 
-			txt=mp_create_txt(MSG_KEYHELP);
+			txt=mp_create_txt(L(MSG_KEYHELP));
 
-			mp_put_str(txt,HELP_TEXT,1);
+			mp_put_str(txt,L(MSG_HELP_TEXT),1);
 			mp_move_bof(txt);
 
 			txt->type=MP_TYPE_READ_ONLY;
@@ -1357,10 +1404,10 @@ int mpi_action(mp_txt * txt, int key)
 	case MPK_TAG:
 
 		mp_get_word(_mp_active,_draw_word,sizeof(_draw_word));
-		if((ptr=mpv_readline(MPR_TAG,MSG_TAGTOSEEK,_draw_word))!=NULL)
+		if((ptr=mpv_readline(MPR_TAG,L(MSG_TAGTOSEEK),_draw_word))!=NULL)
 		{
-			if( ! mpi_find_tag(ptr))
-				mpv_alert(MSG_TAGNOTFOUND,ptr);
+			if(mpi_find_tag(ptr,0)==0)
+				mpv_alert(L(MSG_TAGNOTFOUND),ptr);
 		}
 
 		ret=2;
@@ -1406,7 +1453,7 @@ int mpi_action(mp_txt * txt, int key)
 
 	case MPK_WORDWRAP:
 
-		if((ptr=mpv_readline(MPR_WORDWRAP,MSG_WORDWRAP,NULL))!=NULL)
+		if((ptr=mpv_readline(MPR_WORDWRAP,L(MSG_WORDWRAP),NULL))!=NULL)
 		{
 			_mp_word_wrap=atoi(ptr);
 
@@ -1420,7 +1467,7 @@ int mpi_action(mp_txt * txt, int key)
 
 	case MPK_TABSIZE:
 
-		if((ptr=mpv_readline(MPR_TABSIZE,MSG_TABSIZE,NULL))!=NULL)
+		if((ptr=mpv_readline(MPR_TABSIZE,L(MSG_TABSIZE),NULL))!=NULL)
 		{
 			_mp_tab_size=atoi(ptr);
 
@@ -1434,7 +1481,7 @@ int mpi_action(mp_txt * txt, int key)
 
 	case MPK_TEMPLATE:
 		if(!mpi_insert_template())
-			mpv_alert(MSG_TMPLNOTFOUND,_mpi_template_file);
+			mpv_alert(L(MSG_TMPLNOTFOUND),_mpi_template_file);
 		ret=2;
 
 		break;
@@ -1445,12 +1492,36 @@ int mpi_action(mp_txt * txt, int key)
 		if(mp_load_file(_mp_active,_mpi_template_file)!=-1)
 			mps_auto_synhi(_mp_active);
 		else
-			mp_put_str(_mp_active,EMPTY_TEMPLATE,1);
+			mp_put_str(_mp_active,L(MSG_EMPTY_TEMPLATE),1);
 
 		_mp_active->mod=0;
 		ret=2;
 		break;
 
+	case MPK_COMPLETION:
+
+		{
+			char tmp[128];
+
+			mp_move_left(txt);
+			mp_get_word(txt,tmp,sizeof(tmp));
+			mp_move_right(txt);
+
+			if(mpi_find_tag(tmp,1) > 0)
+			{
+				for(n=strlen(tmp);n>0;n--)
+				{
+					mp_move_left(txt);
+					mp_delete_char(txt);
+				}
+
+				mp_put_str(txt,_draw_word,1);
+			}
+
+			ret=1;
+		}
+		break;
+
 	case MPK_EXIT:
 		_mpi_exit_requested=1;
 		break;
@@ -1463,12 +1534,15 @@ int mpi_action(mp_txt * txt, int key)
 		break;
 	}
 
+	if(mpi_move_selecting)
+		mp_mark(txt);
+
 	if(ret==2)
 	{
 		/* if no text exists, create a new empty one */
 		if(!_mp_active)
 		{
-			mp_create_txt(MSG_UNNAMED);
+			mp_create_txt(L(MSG_UNNAMED));
 			mps_auto_synhi(_mp_active);
 		}
 
@@ -1502,55 +1576,222 @@ int mpi_action(mp_txt * txt, int key)
  */
 void mpi_startup(void)
 {
-	mpv_add_menu(MSG_MENU10);
-	mpv_add_menu_item(MSG_MENU11,MPK_NEW,NULL);
-	mpv_add_menu_item(MSG_MENU12,MPK_OPEN,NULL);
-	mpv_add_menu_item(MSG_MENU13,MPK_REOPEN,NULL);
-	mpv_add_menu_item(MSG_MENU14,MPK_SAVE,NULL);
-	mpv_add_menu_item(MSG_MENU15,MPK_SAVEAS,NULL);
-	mpv_add_menu_item(MSG_MENU16,MPK_CLOSE,NULL);
+	mpv_add_menu(L(MSG_MENU10));
+	mpv_add_menu_item(L(MSG_MENU11),MPK_NEW,NULL);
+	mpv_add_menu_item(L(MSG_MENU12),MPK_OPEN,NULL);
+	mpv_add_menu_item(L(MSG_MENU13),MPK_REOPEN,NULL);
+	mpv_add_menu_item(L(MSG_MENU14),MPK_SAVE,NULL);
+	mpv_add_menu_item(L(MSG_MENU15),MPK_SAVEAS,NULL);
+	mpv_add_menu_item(L(MSG_MENU16),MPK_CLOSE,NULL);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU17,MPK_EXIT,NULL);
-	mpv_add_menu(MSG_MENU20);
-	mpv_add_menu_item(MSG_MENU21,MPK_COPY,NULL);
-	mpv_add_menu_item(MSG_MENU22,MPK_CUT,NULL);
-	mpv_add_menu_item(MSG_MENU23,MPK_PASTE,NULL);
-	mpv_add_menu_item(MSG_MENU24,MPK_DELLINE,NULL);
+	mpv_add_menu_item(L(MSG_MENU17),MPK_EXIT,NULL);
+	mpv_add_menu(L(MSG_MENU20));
+	mpv_add_menu_item(L(MSG_MENU21),MPK_COPY,NULL);
+	mpv_add_menu_item(L(MSG_MENU22),MPK_CUT,NULL);
+	mpv_add_menu_item(L(MSG_MENU23),MPK_PASTE,NULL);
+	mpv_add_menu_item(L(MSG_MENU24),MPK_DELLINE,NULL);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU25,MPK_EDITTMPL,NULL);
+	mpv_add_menu_item(L(MSG_MENU25),MPK_EDITTMPL,NULL);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU26,MPK_EXEC,NULL);
-	mpv_add_menu(MSG_MENU30);
-	mpv_add_menu_item(MSG_MENU31,MPK_SEEK,NULL);
-	mpv_add_menu_item(MSG_MENU32,MPK_SEEKNEXT,NULL);
-	mpv_add_menu_item(MSG_MENU33,MPK_REPLACE,NULL);
-	mpv_add_menu_item(MSG_MENU34,MPK_REPLACE_ALL,NULL);
-	mpv_add_menu_item(MSG_MENU35,MPK_TOGGLECASE,&_mp_case_cmp);
+	mpv_add_menu_item(L(MSG_MENU26),MPK_EXEC,NULL);
+	mpv_add_menu(L(MSG_MENU30));
+	mpv_add_menu_item(L(MSG_MENU31),MPK_SEEK,NULL);
+	mpv_add_menu_item(L(MSG_MENU32),MPK_SEEKNEXT,NULL);
+	mpv_add_menu_item(L(MSG_MENU33),MPK_REPLACE,NULL);
+	mpv_add_menu_item(L(MSG_MENU34),MPK_REPLACE_ALL,NULL);
+	mpv_add_menu_item(L(MSG_MENU35),MPK_TOGGLECASE,&_mp_case_cmp);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU36,MPK_TAG,NULL);
-	mpv_add_menu(MSG_MENU40);
-	mpv_add_menu_item(MSG_MENU41,MPK_NEXT,NULL);
-	mpv_add_menu_item(MSG_MENU42,MPK_BOF,NULL);
-	mpv_add_menu_item(MSG_MENU43,MPK_EOF,NULL);
-	mpv_add_menu_item(MSG_MENU44,MPK_BOL,NULL);
-	mpv_add_menu_item(MSG_MENU45,MPK_EOL,NULL);
-	mpv_add_menu_item(MSG_MENU46,MPK_GOTO,NULL);
-	mpv_add_menu_item(MSG_MENU47,MPK_WORDRIGHT,NULL);
-	mpv_add_menu_item(MSG_MENU48,MPK_WORDLEFT,NULL);
+	mpv_add_menu_item(L(MSG_MENU36),MPK_TAG,NULL);
+	mpv_add_menu(L(MSG_MENU40));
+	mpv_add_menu_item(L(MSG_MENU41),MPK_NEXT,NULL);
+	mpv_add_menu_item(L(MSG_MENU42),MPK_BOF,NULL);
+	mpv_add_menu_item(L(MSG_MENU43),MPK_EOF,NULL);
+	mpv_add_menu_item(L(MSG_MENU44),MPK_BOL,NULL);
+	mpv_add_menu_item(L(MSG_MENU45),MPK_EOL,NULL);
+	mpv_add_menu_item(L(MSG_MENU46),MPK_GOTO,NULL);
+	mpv_add_menu_item(L(MSG_MENU47),MPK_WORDRIGHT,NULL);
+	mpv_add_menu_item(L(MSG_MENU48),MPK_WORDLEFT,NULL);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU49,MPK_OPEN2,NULL);
-	mpv_add_menu(MSG_MENU50);
-	mpv_add_menu_item(MSG_MENU51,MPK_SAVETABS,&_mp_save_tabs);
-	mpv_add_menu_item(MSG_MENU52,MPK_CRLF,&_mp_cr_lf);
-	mpv_add_menu_item(MSG_MENU53,MPK_AUTOINDENT,&_mp_auto_indent);
-	mpv_add_menu_item(MSG_MENU54,MPK_COL80,&_mpi_mark_column_80);
+	mpv_add_menu_item(L(MSG_MENU49),MPK_OPEN2,NULL);
+	mpv_add_menu(L(MSG_MENU50));
+	mpv_add_menu_item(L(MSG_MENU51),MPK_SAVETABS,&_mp_save_tabs);
+	mpv_add_menu_item(L(MSG_MENU52),MPK_CRLF,&_mp_cr_lf);
+	mpv_add_menu_item(L(MSG_MENU53),MPK_AUTOINDENT,&_mp_auto_indent);
+	mpv_add_menu_item(L(MSG_MENU54),MPK_COL80,&_mpi_mark_column_80);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU55,MPK_TABSIZE,NULL);
-	mpv_add_menu_item(MSG_MENU56,MPK_WORDWRAP,NULL);
+	mpv_add_menu_item(L(MSG_MENU55),MPK_TABSIZE,NULL);
+	mpv_add_menu_item(L(MSG_MENU56),MPK_WORDWRAP,NULL);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU57,MPK_KEYHELP,NULL);
+	mpv_add_menu_item(L(MSG_MENU57),MPK_KEYHELP,NULL);
 	mpv_add_menu_item("-",0,NULL);
-	mpv_add_menu_item(MSG_MENU58,MPK_ABOUT,NULL);
+	mpv_add_menu_item(L(MSG_MENU58),MPK_ABOUT,NULL);
 
 	_mpi_macro[0]=MPK_RECORDMACRO;
 }
+
+
+/**
+ * mpi_shutdown - Shutdown interface function.
+ *
+ * Shuts down the interface.
+ */
+void mpi_shutdown(void)
+{
+	/* closes all open texts */
+	while(_mp_active)
+	{
+		mpi_close(_mp_active);
+
+		if(_mp_active)
+		{
+			mpi_draw_all(_mp_active);
+			mpv_title(NULL);
+		}
+	}
+}
+
+
+/**
+ * mpi_args_1 - Command line argument processing, 1st pass
+ * @argc: argument count
+ * @argv: the arguments
+ *
+ * First pass to argument processing, mainly on/off switches.
+ * Returns -1 if usage printing needed, -2 if version number,
+ * 0 otherwise.
+ */
+int mpi_args_1(int argc, char * argv[])
+{
+	int n;
+	char * lang;
+
+	/* if the LANG or LC_ALL environment variable is defined,
+		use it as default language */
+	if((lang=getenv("LANG"))!=NULL || (lang=getenv("LC_ALL"))!=NULL)
+		mpl_set_language(lang);
+
+	/* first pass: general switches, previous to initialization */
+	for(n=1;n < argc;n++)
+	{
+		if(strcmp(argv[n],"--help")==0 ||
+		   strcmp(argv[n],"-h")==0)
+			return(-1);
+		else
+		if(strcmp(argv[n],"--version-only")==0 ||
+		   strcmp(argv[n],"--version")==0)
+			return(-2);
+		else
+		if(strcmp(argv[n],"--alt-color")==0)
+			mpi_alt_color^=1;
+		else
+		if(strcmp(argv[n],"--mouse")==0)
+			mpi_mouse=1;
+		else
+		if(strcmp(argv[n],"--col80")==0)
+			_mpi_mark_column_80=1;
+		else
+		if(strcmp(argv[n],"--hardware-cursor")==0 ||
+		   strcmp(argv[n],"-hw")==0)
+			_mpi_reposition_cursor=1;
+		else
+		if(strcmp(argv[n],"--monochrome")==0 ||
+		   strcmp(argv[n],"-bw")==0)
+			mpi_monochrome=1;
+		else
+		if(strcmp(argv[n],"--autoindent")==0 ||
+		   strcmp(argv[n],"-ai")==0)
+			_mp_auto_indent^=1;
+		else
+		if(strcmp(argv[n],"--no-transparent")==0 ||
+		   strcmp(argv[n],"-nt")==0)
+			mpi_transp_mode^=1;
+		else
+		if(strcmp(argv[n],"-l")==0 ||
+		   strcmp(argv[n],"--lang")==0)
+		{
+			if(n < argc-1)
+			{
+				n++;
+				mpl_set_language(argv[n]);
+			}
+		}
+	}
+
+	return(0);
+}
+
+
+/**
+ * mpi_args_2 - Command line argument processing, 2nd pass
+ * @argc: argument count
+ * @argv: the arguments
+ *
+ * Second pass to argument processing, commands and files to load.
+ * Returns -1 if an invalid mode is requested, -2 if non-
+ * existing tags are requested, 0 otherwise.
+ */
+int mpi_args_2(int argc, char * argv[])
+{
+	int n,bad_tags;
+
+	bad_tags=0;
+
+	/* second pass: switches with args and files */
+	for(n=1;n < argc;n++)
+	{
+		if(strcmp(argv[n],"-t")==0 ||
+		   strcmp(argv[n],"--tag")==0)
+		{
+			if(n < argc - 1)
+			{
+				n++;
+				if(mpi_find_tag(argv[n],0)==0)
+					bad_tags++;
+			}
+		}
+		else
+		if(strcmp(argv[n],"-w")==0 ||
+		   strcmp(argv[n],"--word-wrap")==0)
+		{
+			if(n < argc - 1)
+			{
+				n++;
+				_mp_word_wrap=atoi(argv[n]);
+			}
+		}
+		else
+		if(strcmp(argv[n],"--tab-size")==0 ||
+		   strcmp(argv[n],"-ts")==0)
+		{
+			if(n < argc-1)
+			{
+				n++;
+				_mp_tab_size=atoi(argv[n]);
+			}
+		}
+		else
+		if(strcmp(argv[n],"-m")==0 ||
+		   strcmp(argv[n],"--mode")==0)
+		{
+			if(n < argc-1)
+			{
+				n++;
+				if(! mps_set_override_mode(argv[n]))
+					return(-1);
+			}
+		}
+		else
+		if(strcmp(argv[n],"-l")==0 ||
+		   strcmp(argv[n],"--lang")==0)
+		{
+			n++;
+		}
+		else
+		if(argv[n][0]!='-')
+			mpi_open(argv[n],0);
+	}
+
+	if(bad_tags) return(-2);
+	return(0);
+}
+
