@@ -209,6 +209,12 @@ static char * _php_helpers[] = { "man %s", NULL };
 
 static char * _plain_magics_1[] = { "-*- Mode: plain", NULL };
 
+/* .po (gettext) files */
+
+static char * _po_exts[] = { ".po", ".pot", ".pox", NULL };
+static char * _po_magics_1[] = { "-*- Mode: po", NULL };
+static char * _po_magics_2[] = { "msgid", "msgstr", NULL };
+static char * _po_tokens[] = { "msgid", "msgstr", NULL };
 
 
 struct mps_synhi _mps_synhi[]=
@@ -265,6 +271,11 @@ struct mps_synhi _mps_synhi[]=
 		_php_magics_1, _php_magics_2, _php_exts, _php_tokens,
 		_php_vars, _php_helpers, NULL, 1, 0  },
 
+	/* po files */
+	{ "po", "po", "\"`", "\"`", "#", NULL,
+		_po_magics_1, _po_magics_2, _po_exts, _po_tokens,
+		NULL, NULL, NULL, 0, 0 },
+
 	/* ... yours here ... */
 
 	/* End of syntax hilighters */
@@ -665,7 +676,7 @@ char * mps_enumerate_modes(void)
 	static char modes[1024];
 	int n;
 
-	/* buffer overflow would be tested */
+	/* buffer overflow should be tested */
 
 	modes[0]='\0';
 	for(n=0;_mps_synhi[n].type!=NULL;n++)
