@@ -45,6 +45,7 @@
 #include "mp_synhi.h"
 #include "mp_iface.h"
 
+#include "mp_lang.h"
 
 /*******************
 	Data
@@ -64,6 +65,7 @@ static int _title_attr;
 static int _menu_sel_attr;
 static int _menu_act_attr;
 static int _menu_frame_attr;
+static int _menu_frame2_attr;
 static int _scrollbar_attr;
 static int _scrollbar_thumb_attr;
 static int _tab_sel_attr;
@@ -102,26 +104,10 @@ struct mpv_menu_bar * _mpv_menu_last=NULL;
 /* menu bar horizontal position */
 static int _mpv_menu_xpos=2;
 
-/* alternate color set */
-#ifndef ALT_COLOR
-#define ALT_COLOR 0
-#endif
-int _alt_color=ALT_COLOR;
-
-
-/* use mouse to position cursor or not */
-int _mouse=0;
-
-/* monochrome mode */
-int _monochrome=0;
-
 #ifndef NCURSES_VERSION
 #define NO_TRANSPARENCY
 #endif
 
-/* use transparent mode (if applicable) */
-int _transp_mode=1;
-
 
 /* history for readlines */
 
@@ -148,9 +134,27 @@ struct readhist
 char * _mpv_hint_text=NULL;
 
 
-/* ***************
-	Code
-   *************** */
+#ifdef POOR_MAN_BOXES
+#define MPBOX_HLINE '-'
+#define MPBOX_VLINE '|'
+#define MPBOX_TL_CORNER '+'
+#define MPBOX_TR_CORNER '+'
+#define MPBOX_BL_CORNER '+'
+#define MPBOX_BR_CORNER '+'
+#define MPBOX_THUMB ' '
+#else
+#define MPBOX_HLINE ACS_HLINE
+#define MPBOX_VLINE ACS_VLINE
+#define MPBOX_TL_CORNER ACS_ULCORNER
+#define MPBOX_TR_CORNER ACS_URCORNER
+#define MPBOX_BL_CORNER ACS_LLCORNER
+#define MPBOX_BR_CORNER ACS_LRCORNER
+#define MPBOX_THUMB ACS_CKBOARD
+#endif
+
+/*****************
+       Code
+******************/
 
 static void _mpv_attrset(int attr)
 {
@@ -320,7 +324,7 @@ static void _mpv_title_status(int y, char * str1, char * str2)
 			addch(*str2);
 	}
 
-	if(_mpv_hint_text==NULL) _mpv_hint_text=MSG_DEFAULTHINT;
+	if(_mpv_hint_text==NULL) _mpv_hint_text=L(MSG_DEFAULTHINT);
 
 	if(y!=0)
 	{
@@ -399,7 +403,7 @@ void mpv_title(char * str)
  */
 void mpv_status_line(char * str)
 {
-	_mpv_title_status(LINES-1,str,NULL);
+	_mpv_title_status(LINES - 1,str,NULL);
 }
 
 
@@ -506,9 +510,9 @@ static int _mpv_key(int k)
 	case ctrl('V'): k=MPK_PASTE; break;
 	case ctrl('U'): k=MPK_TEMPLATE; break;
 	case ctrl('F'): k=MPK_TAG; break;
+	case ctrl('S'): k=MPK_COMPLETION; break;
 
 	case KEY_F(1):	k=MPK_HELP; break;
-	case ctrl('S'):
 	case KEY_F(2):	k=MPK_SAVE; break;
 	case KEY_F(3):	k=MPK_OPEN; break;
 	case KEY_F(4):	k=MPK_CLOSE; break;
@@ -663,6 +667,33 @@ void mpv_add_menu_item(char * label, int key, int * toggle)
 }
 
 
+static void _mpv_box(WINDOW * w, int tx, int ty)
+{
+	int x,y;
+
+	wmove(w,0,0);
+	waddch(w,MPBOX_TL_CORNER);
+	for(x=1;x < tx-1;x++)
+		waddch(w,MPBOX_HLINE);
+	waddch(w,MPBOX_TR_CORNER);
+
+	for(y=1;y < ty-1;y++)
+	{
+		_mpv_wattrset(w,_menu_frame_attr);
+		wmove(w,y,0);
+		waddch(w,MPBOX_VLINE);
+		_mpv_wattrset(w,_menu_frame2_attr);
+		wmove(w,y,tx-1);
+		waddch(w,MPBOX_VLINE);
+	}
+
+	waddch(w,MPBOX_BL_CORNER);
+	for(x=1;x < tx-1;x++)
+		waddch(w,MPBOX_HLINE);
+	waddch(w,MPBOX_BR_CORNER);
+}
+
+
 /**
  * mpv_menu - Manages the menu.
  *
@@ -687,7 +718,7 @@ int mpv_menu(void)
 		strcat(tmp,m->label);
 	}
 
-	_mpv_hint_text=MSG_CANCELHINT;
+	_mpv_hint_text=L(MSG_CANCELHINT);
 	_mpv_title_status(0,tmp,NULL);
 	m=_mpv_menu;
 
@@ -698,7 +729,8 @@ int mpv_menu(void)
 
 		w=newwin(m->items+2, sizeof(i->label), 1, m->xpos);
 		_mpv_wattrset(w,_menu_frame_attr);
-		box(w,0,0);
+/*		  box(w,0,0); */
+		_mpv_box(w,sizeof(i->label),m->items+2);
 		wrefresh(w);
 
 		while(!ok)
@@ -717,13 +749,13 @@ int mpv_menu(void)
 				/* if '-', then item is a separator */
 				if(i2->label[0]=='-')
 				{
-					wmove(w,y,0);
+					wmove(w,y,1);
 					_mpv_wattrset(w,_menu_frame_attr);
 
-					waddch(w,ACS_LTEE);
+					/* waddch(w,ACS_LTEE); */
 					for(x=0;x<sizeof(i2->label)-2;x++)
-						waddch(w,ACS_HLINE);
-					waddch(w,ACS_RTEE);
+						waddch(w,MPBOX_HLINE);
+					/* waddch(w,ACS_RTEE); */
 				}
 				else
 				{
@@ -734,7 +766,7 @@ int mpv_menu(void)
 					{
 						if(cc=='&')
 						{
-							_mpv_wattrset(w,_menu_act_attr);
+							/* _mpv_wattrset(w,_menu_act_attr); */
 							cc=i2->label[++x];
 						}
 						else
@@ -832,26 +864,31 @@ int mpv_menu(void)
 static int _mpv_prompt(char * prompt, char * prompt2)
 {
 	int c;
+	char * yes;
+	char * no;
 
-	move(LINES-1,0);
+	move(LINES - 1,0);
 	_mpv_attrset(_title_attr);
 
 	addstr(prompt);
 	if(prompt2)
 		addstr(prompt2);
 
+	yes=L(MSG_YES);
+	no=L(MSG_NO);
+
 	for(;;)
 	{
 		c=toupper(getch());
 
-		if(c==MSG_YES || c==MSG_NO || c=='\r')
+		if(c==*yes || c==*no || c=='\r')
 		{
 			mpv_status_line(NULL);
 			break;
 		}
 	}
 
-	if(c==MSG_NO) return(0);
+	if(c==*no) return(0);
 
 	return(1);
 }
@@ -866,7 +903,7 @@ static int _mpv_prompt(char * prompt, char * prompt2)
  */
 int mpv_confirm(char * prompt)
 {
-	return(_mpv_prompt(prompt,MSG_YESNO));
+	return(_mpv_prompt(prompt,L(MSG_YESNO)));
 }
 
 
@@ -887,7 +924,7 @@ void mpv_alert(char * msg, char * msg2)
 	else
 		sprintf(tmp,msg,msg2);
 
-	_mpv_prompt(tmp,MSG_ENTER);
+	_mpv_prompt(tmp,L(MSG_ENTER));
 }
 
 
@@ -905,7 +942,7 @@ int mpv_list(char * title, mp_txt * txt)
 {
 	int c;
 
-	_mpv_hint_text=MSG_CANCELHINT;
+	_mpv_hint_text=L(MSG_CANCELHINT);
 	mpv_title(title);
 
 	mp_move_bof(txt);
@@ -914,6 +951,7 @@ int mpv_list(char * title, mp_txt * txt)
 	if(txt->lasty==0)
 	{
 		/* no lines or just one line: exit */
+		_mpv_hint_text=NULL;
 		return(0);
 	}
 
@@ -1128,11 +1166,13 @@ char * mpv_readline(int type, char * prompt, char * def)
 		else
 		{
 			struct stat s;
-			stat(tmp,&s);
 
-			/* add a globber if it's a directory */
-			if(s.st_mode & S_IFDIR)
-				strcat(tmp,"/*");
+			if(!stat(tmp,&s))
+			{
+				/* add a globber if it's a directory */
+				if(s.st_mode & S_IFDIR)
+					strcat(tmp,"/*");
+			}
 		}
 
 		if(strchr(tmp,'*')!=NULL || strchr(tmp,'?')!=NULL)
@@ -1178,7 +1218,7 @@ int mpv_help(char * term, int synhi)
 	mp_txt * txt;
 	char ** ptr;
 
-	sprintf(tmp,MSG_HELP,term);
+	sprintf(tmp,L(MSG_HELP),term);
 
 	if(synhi==0 || (txt=mp_create_txt(tmp))==NULL)
 		return(0);
@@ -1202,7 +1242,7 @@ int mpv_help(char * term, int synhi)
 	if(ptr==NULL || *ptr==NULL)
 	{
 		mp_delete_txt(txt);
-		mpv_alert(MSG_NOHELPFOR,term);
+		mpv_alert(L(MSG_NOHELPFOR),term);
 		return(0);
 	}
 
@@ -1224,7 +1264,7 @@ void mpv_about(void)
 {
 	mp_txt * txt;
 
-	txt=mp_create_txt(MSG_ABOUT);
+	txt=mp_create_txt(L(MSG_ABOUT));
 
 	mp_put_str(txt,MP_LICENSE,1);
 	mp_move_bof(txt);
@@ -1283,27 +1323,15 @@ void mpv_scrollbar(int pos, int size, int max)
 		move(START_LINE+n,_mpv_x_size);
 
 		if(n>=l && n<=u && !a)
-#ifdef POOR_MAN_BOXES
 		{
 			_mpv_attrset(_scrollbar_thumb_attr);
-			addch(' ');
+			addch(MPBOX_THUMB);
 		}
 		else
 		{
 			_mpv_attrset(_scrollbar_attr);
-			addch('|');
+			addch(MPBOX_VLINE);
 		}
-#else
-		{
-			_mpv_attrset(_scrollbar_thumb_attr);
-			addch(ACS_CKBOARD);
-		}
-		else
-		{
-			_mpv_attrset(_scrollbar_attr);
-			addch(ACS_VLINE);
-		}
-#endif
 	}
 }
 
@@ -1327,33 +1355,11 @@ void mpv_filetabs(void)
 
 void mpv_xonxoff(int state)
 {
-	static int tty = -1;
 	static struct termios _ioval;
-	char *ttyn;
 	static int orig_state=-1;
 
-	if ( tty == -1 ) {
-
-		if (isatty(0)) {
-			ttyn = ttyname(0);
-		} else {
-			ttyn = "/dev/tty";
-		}
-
-		tty = open(ttyn, O_RDWR);
-
-		if (tty < 0) {
-			tty = 2;
-		}
-	}
-
-	if (tty == -1) {
-		printf("Uninitialised term controller.\n");
-		exit(EXIT_FAILURE);
-	}
-
 	/* Get current settings */
-	tcgetattr(tty, &_ioval);
+	tcgetattr(0, &_ioval);
 
 	/* Store XON/XOFF state */
 	if ( orig_state == -1 ) {
@@ -1365,7 +1371,7 @@ void mpv_xonxoff(int state)
 		_ioval.c_lflag &= ~TTY_MODE;
 		_ioval.c_iflag &= ~(IXON | IXOFF);
 
-		tcsetattr(tty, TCSANOW, &_ioval);
+		tcsetattr(0, TCSANOW, &_ioval);
 		return;
 
 	} else {
@@ -1377,7 +1383,7 @@ void mpv_xonxoff(int state)
 		_ioval.c_lflag |= TTY_MODE;
 		_ioval.c_iflag |= orig_state;
 
-		tcsetattr(tty, TCSANOW, &_ioval);
+		tcsetattr(0, TCSANOW, &_ioval);
 		return;
 	}
 }
@@ -1402,16 +1408,16 @@ int mpv_startup(void)
 #ifndef NO_TRANSPARENCY
 	use_default_colors();
 #else
-	_transp_mode=0;
+	mpi_transp_mode=0;
 #endif
 
 	dic=DEFAULT_INK_COLOR;
 	dpc=DEFAULT_PAPER_COLOR;
 
-	if(_transp_mode) dic=dpc=-1;
+	if(mpi_transp_mode) dic=dpc=-1;
 
 #ifdef NCURSES_MOUSE_VERSION
-	if(_mouse)
+	if(mpi_mouse)
 		mousemask(BUTTON1_PRESSED|BUTTON2_PRESSED, NULL);
 #endif
 
@@ -1420,7 +1426,7 @@ int mpv_startup(void)
 	cbreak();
 	noecho();
 
-	if(_monochrome)
+	if(mpi_monochrome)
 	{
 		init_pair(MP_COLOR_NORMAL,dic,dpc);
 		init_pair(MP_COLOR_SELECTED,dic,dpc);
@@ -1434,6 +1440,8 @@ int mpv_startup(void)
 
 		init_pair(10,dic,dpc);
 		init_pair(11,dic,dpc);
+		init_pair(12,dic,dpc);
+		init_pair(13,dic,dpc);
 	}
 	else
 	{
@@ -1447,11 +1455,13 @@ int mpv_startup(void)
 		init_pair(MP_COLOR_CAPS,COLOR_YELLOW,dpc);
 		init_pair(MP_COLOR_LOCAL,COLOR_CYAN,dpc);
 
-		init_pair(10,_alt_color ? COLOR_RED:COLOR_BLUE,COLOR_WHITE);
-		init_pair(11,_alt_color ? COLOR_RED:COLOR_BLUE,
-			     _alt_color ? COLOR_YELLOW:COLOR_WHITE);
+		init_pair(10,mpi_alt_color ? COLOR_RED:COLOR_BLUE,COLOR_WHITE);
+		init_pair(11,mpi_alt_color ? COLOR_RED:COLOR_BLUE,
+			     mpi_alt_color ? COLOR_YELLOW:COLOR_BLUE);
 		init_pair(12, COLOR_WHITE,
-			     _alt_color ? COLOR_MAGENTA:COLOR_BLACK);
+			     mpi_alt_color ? COLOR_BLUE:COLOR_BLACK);
+		init_pair(13,mpi_alt_color ? COLOR_RED:COLOR_BLUE,
+			     mpi_alt_color ? COLOR_BLACK:COLOR_BLACK);
 	}
 
 	_attrs[MP_COLOR_NORMAL]=COLOR_PAIR(MP_COLOR_NORMAL);
@@ -1466,6 +1476,7 @@ int mpv_startup(void)
 
 	_title_attr=COLOR_PAIR(10)|A_BOLD|A_REVERSE;
 	_menu_frame_attr=COLOR_PAIR(11)|A_BOLD|A_REVERSE;
+	_menu_frame2_attr=COLOR_PAIR(13)|A_BOLD|A_REVERSE;
 	_menu_sel_attr=COLOR_PAIR(12)|A_BOLD;
 	_menu_act_attr=_menu_frame_attr;
 	_scrollbar_attr=COLOR_PAIR(MP_COLOR_NORMAL);
@@ -1527,125 +1538,42 @@ void usage(void)
 	printf("%s\n", __DATE__ " " __TIME__);
 	printf("This software is covered by the GPL license. NO WARRANTY.\n\n");
 
-	printf("%s", USAGE_TEXT);
+	printf("%s", L(MSG_USAGE_TEXT));
 	printf("%s\n",mps_enumerate_modes());
 }
 
 
 int main(int argc, char * argv[])
 {
-	int c, n=1;
-	int bad_tags=0;
+	int c, r;
 
-	/* first pass: general switches, previous to initialization */
-	for(n=1;n<argc;n++)
-	{
-		if(strcmp(argv[n],"--help")==0 ||
-		   strcmp(argv[n],"-h")==0)
-		{
-			usage();
-			exit(0);
-		}
-		else
-		if(strcmp(argv[n],"--version-only")==0 ||
-		   strcmp(argv[n],"--version")==0)
-		{
-			printf(VERSION "\n");
-			exit(0);
-		}
-		else
-		if(strcmp(argv[n],"--alt-color")==0)
-			_alt_color^=1;
-		else
-		if(strcmp(argv[n],"--mouse")==0)
-			_mouse=1;
-		else
-		if(strcmp(argv[n],"--col80")==0)
-			_mpi_mark_column_80=1;
-		else
-		if(strcmp(argv[n],"--hardware-cursor")==0 ||
-		   strcmp(argv[n],"-hw")==0)
-			_mpi_reposition_cursor=1;
-		else
-		if(strcmp(argv[n],"--monochrome")==0 ||
-		   strcmp(argv[n],"-bw")==0)
-			_monochrome=1;
-		else
-		if(strcmp(argv[n],"--autoindent")==0 ||
-		   strcmp(argv[n],"-ai")==0)
-			_mp_auto_indent^=1;
-		else
-		if(strcmp(argv[n],"--no-transparent")==0 ||
-		   strcmp(argv[n],"-nt")==0)
-			_transp_mode^=1;
-	}
+	r=mpi_args_1(argc, argv);
+
+	if(r==-1) { usage(); exit(0); }
+	if(r==-2) { printf(VERSION "\n"); exit(0); }
 
 	mp_startup();
 	mps_startup();
 	mpi_startup();
 	mpv_startup();
 
-	/* second pass: switches with args and files */
-	for(n=1;n<argc;n++)
+	r=mpi_args_2(argc, argv);
+
+	if(r==-1)
 	{
-		if(strcmp(argv[n],"-t")==0 ||
-		   strcmp(argv[n],"--tag")==0)
-		{
-			if(n<argc-1)
-			{
-				n++;
-				if(!mpi_find_tag(argv[n]))
-					bad_tags++;
-			}
-		}
-		else
-		if(strcmp(argv[n],"-w")==0 ||
-		   strcmp(argv[n],"--word-wrap")==0)
-		{
-			if(n<argc-1)
-			{
-				n++;
-				_mp_word_wrap=atoi(argv[n]);
-			}
-		}
-		else
-		if(strcmp(argv[n],"--tab-size")==0 ||
-		   strcmp(argv[n],"-ts")==0)
-		{
-			if(n<argc-1)
-			{
-				n++;
-				_mp_tab_size=atoi(argv[n]);
-			}
-		}
-		else
-		if(strcmp(argv[n],"-m")==0 ||
-		   strcmp(argv[n],"--mode")==0)
-		{
-			if(n<argc-1)
-			{
-				n++;
-				if(!mps_set_override_mode(argv[n]))
-				{
-					mpv_shutdown();
-					printf("%s\n",MSG_BADMODE);
-					printf("%s\n",mps_enumerate_modes());
-					exit(1);
-				}
-			}
-		}
-		else
-		if(argv[n][0]!='-')
-			mpi_open(argv[n],0);
+		mpv_shutdown();
+		printf("%s\n",L(MSG_BADMODE));
+		printf("%s\n",mps_enumerate_modes());
+		exit(1);
 	}
 
-	/* force the reading of the tags file, if exists */
-	mpi_find_tag(NULL);
+	/* force the reading of the tags file, if one exists */
+	mpi_find_tag(NULL,0);
 
 	/* create empty text if no file is open */
 	if(_mp_active==NULL)
 	{
-		mp_create_txt(MSG_UNNAMED);
+		mp_create_txt(L(MSG_UNNAMED));
 		mps_auto_synhi(_mp_active);
 	}
 
@@ -1653,9 +1581,9 @@ int main(int argc, char * argv[])
 	mpi_draw_all(_mp_active);
 
 	/* bark if bad tags */
-	if(bad_tags)
+	if(r==-2)
 	{
-		mpv_alert(MSG_TAGNOTFOUND,"");
+		mpv_alert(L(MSG_TAGNOTFOUND),"");
 		mpi_draw_all(_mp_active);
 	}
 
@@ -1680,16 +1608,7 @@ int main(int argc, char * argv[])
 	}
 
 	/* close everything */
-	while(_mp_active)
-	{
-		mpi_close(_mp_active);
-
-		if(_mp_active)
-		{
-			mpi_draw_all(_mp_active);
-			mpv_title(NULL);
-		}
-	}
+	mpi_shutdown();
 
 	mpv_title(NULL);
 	mpv_refresh();
