@@ -32,7 +32,9 @@
 
 #include "mp_core.h"
 #include "mp_video.h"
+#include "mp_lang.h"
 #include "mp_iface.h"
+#include "mp_synhi.h"
 #include "mp_res.h"
 
 
@@ -100,7 +102,13 @@ static HMENU _smenu=NULL;
 static char _menu_label[40]="";
 
 /* height of the tab of files */
-int _tab_height=32;
+int _tab_height=28;
+
+static int _mpv_last_action=2;
+
+/* microsoft moronity overriding */
+int _argc;
+char * _argv[100];
 
 
 /*******************
@@ -481,7 +489,7 @@ BOOL CALLBACK AboutDlgProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
 	{
 	case WM_INITDIALOG:
 
-		SetWindowText(hwnd, MSG_ABOUT);
+		SetWindowText(hwnd, L(MSG_ABOUT));
 
 		/* move the license text converting \n to \r\n */
 		for(n=0,ptr=tmp;MP_LICENSE[n];n++)
@@ -905,6 +913,8 @@ static int mpv_vkey(int c)
 	   GetKeyState(VK_MENU)&0x8000)
 		ctrl=1;
 
+	mpi_move_selecting=(GetKeyState(VK_SHIFT) & 0x8000);
+
 	if(_mpi_shift)
 		_mpi_shift=0;
 
@@ -997,6 +1007,7 @@ static int mpv_akey(int k)
 	case _ctrl('O'): k=MPK_OPEN2; break;
 	case _ctrl('U'): k=MPK_TEMPLATE; break;
 	case _ctrl('F'): k=MPK_TAG; break;
+	case _ctrl('S'): k=MPK_COMPLETION; break;
 
 	case _ctrl('N'): k=MPK_SHIFT; break;
 
@@ -1020,7 +1031,7 @@ static void _mpv_action(int c)
 	if(c==MPK_PASTE)
 		_windows_to_clpbrd();
 
-	if(mpi_action(_mp_active,c))
+	if((_mpv_last_action=mpi_action(_mp_active,c)))
 		mpi_draw_all(_mp_active);
 
 	/* post actions */
@@ -1146,8 +1157,7 @@ long STDCALL WndProc(HWND hwnd, UINT msg, UINT wparam, LONG lparam)
 		return(0);
 
 	case WM_CLOSE:
-		while(_mp_active)
-			mpi_close(_mp_active);
+		mpi_shutdown();
 
 		DestroyWindow(hwnd);
 		return(0);
@@ -1188,9 +1198,9 @@ long STDCALL WndProc(HWND hwnd, UINT msg, UINT wparam, LONG lparam)
  * @size: vertical size
  * @max: maximum value
  *
- * Draws / updates the scrollbar. pos is current position
- * (the line where the cursor is), size is the number of
- * lines of the document window and max is the maximum
+ * Draws / updates the scrollbar. @pos is current position
+ * (the line where the cursor is), @size is the number of
+ * lines of the document window and @max is the maximum
  * value (the total number of lines in current text).
  */
 void mpv_scrollbar(int pos, int size, int max)
@@ -1220,6 +1230,8 @@ void mpv_filetabs(void)
 	int n;
 	char * ptr;
 
+	if(_mpv_last_action!=2) return;
+
 	TabCtrl_DeleteAllItems(hwtabs);
 
 	for(t=_mp_txts,n=0;t!=NULL;t=t->next,n++)
@@ -1241,6 +1253,47 @@ void mpv_filetabs(void)
 }
 
 
+static void set_system_language(void)
+{
+	short s;
+	char * ptr;
+
+	s=GetSystemDefaultLangID() & 0x00ff;
+
+	/* try only some 'famous' languages:
+	   the complete world language database should
+	   be implemented */
+	switch(s)
+	{
+	case 0x01: ptr="ar"; break; /* arabic */
+	case 0x02: ptr="bg"; break; /* bulgarian */
+	case 0x03: ptr="ca"; break; /* catalan */
+	case 0x04: ptr="zh"; break; /* chinese */
+	case 0x05: ptr="cz"; break; /* czech */
+	case 0x06: ptr="da"; break; /* danish */
+	case 0x07: ptr="de"; break; /* german */
+	case 0x08: ptr="el"; break; /* greek */
+	case 0x09: ptr="en"; break; /* english */
+	case 0x0a: ptr="es"; break; /* spanish */
+	case 0x0b: ptr="fi"; break; /* finnish */
+	case 0x0c: ptr="fr"; break; /* french */
+	case 0x0d: ptr="he"; break; /* hebrew */
+	case 0x0e: ptr="hu"; break; /* hungarian */
+	case 0x0f: ptr="is"; break; /* icelandic */
+	case 0x10: ptr="it"; break; /* italian */
+	case 0x11: ptr="jp"; break; /* japanese */
+	case 0x13: ptr="nl"; break; /* dutch */
+	case 0x14: ptr="no"; break; /* norwegian */
+	case 0x15: ptr="po"; break; /* polish */
+	case 0x16: ptr="pt"; break; /* portuguese */
+	case 0x1d: ptr="se"; break; /* swedish */
+	default: ptr="en"; break;
+	}
+
+	mpl_set_language(ptr);
+}
+
+
 /**
  * mpv_startup - Starts up the system dependent driver.
  *
@@ -1254,6 +1307,8 @@ int mpv_startup(void)
 	HKEY hkey;
 	RECT r;
 
+	InitCommonControls();
+
 	/* fill a usable default template file */
 	strcpy(_mpi_template_file,"C:\\mp_templates");
 
@@ -1322,6 +1377,9 @@ int mpv_startup(void)
 		WS_CHILD | TCS_TABS | TCS_SINGLELINE | TCS_FOCUSNEVER,
 		0, 0, r.right-r.left, _tab_height, hwnd, NULL, hinst, NULL);
 
+	SendMessage(hwtabs, WM_SETFONT, 
+		(WPARAM) GetStockObject(ANSI_VAR_FONT), 0);
+
 	ShowWindow(hwtabs,SW_SHOW);
 	UpdateWindow(hwtabs);
 
@@ -1379,25 +1437,72 @@ void mpv_shutdown(void)
 }
 
 
+static void cmd_2_argv(char * cmd)
+{
+	char d;
+	_argc=1;
+
+	d=' ';
+	while(*cmd)
+	{
+		while(*cmd==d || *cmd==' ')
+		{
+			*cmd='\0';
+			cmd++;
+		}
+
+		if(*cmd=='"' || *cmd=='\'')
+		{
+			d=*cmd;
+			cmd++;
+		}
+		else
+			d=' ';
+
+		_argv[_argc]=cmd;
+		_argc++;
+
+		while(*cmd && *cmd!=' ')
+			cmd++;
+	}
+}
+
+
 int STDCALL WinMain(HINSTANCE hi, HINSTANCE hpi, LPSTR cmd, int cm)
 {
-	char * f;
+	int r;
+
 	hinst=hi;
 
+	cmd_2_argv(cmd);
+	set_system_language();
+
+	r=mpi_args_1(_argc, _argv);
+
+	if(r < 0) { exit(0); }
+
 	mp_startup();
 	mps_startup();
 	mpi_startup();
 	mpv_startup();
 
-	/* load everything sent to command line, splitting it */
-	for(f=strtok(cmd," ");f!=NULL;f=strtok(NULL," "))
-		mpi_open(f,0);
+	r=mpi_args_2(_argc, _argv);
 
-	/* force the reading of the tags file, if exists */
-	mpi_find_tag(NULL);
+	if(r==-1)
+	{
+		mpv_alert(mps_enumerate_modes(),NULL);
+		mpv_shutdown();
+		exit(1);
+	}
+
+	/* force the reading of the tags file, if one exists */
+	mpi_find_tag(NULL,0);
 
 	if(_mp_active==NULL)
-		mp_create_txt(MSG_UNNAMED);
+	{
+		mp_create_txt(L(MSG_UNNAMED));
+		mps_auto_synhi(_mp_active);
+	}
 
 	mpv_title(_mp_active->name);
 	mpi_draw_all(_mp_active);
